USE master
GO

--clr_enabled is an advanced option
sp_configure 'show advanced options', 1
GO
RECONFIGURE
GO

--'clr enabled' is mandatory when using CLR
sp_configure 'clr enabled', 1
GO
RECONFIGURE
GO

--new option starting from 2017 see https://www.mobilo24.eu/sql-clr-w-wersji-2017opcja-clr-strict-security/ for better description
sp_configure 'clr strict security'
GO

--starting from SQL 2017 all assemblies should be signed by certificate (note the validity date) or with assymetric key
--only public key is required
--see https://www.mobilo24.eu/sql-clr-podpisywanie-kodu/ on how to sign code
CREATE ASYMMETRIC KEY Sun_ASYMETRICKEY FROM EXECUTABLE  FILE = 'C:\temp\SunClr.dll' 
GO

SELECT * FROM sys.asymmetric_keys
GO

--the asymmetric key should be associated with login having "UNSAFE ASSEMBLY" permission
CREATE LOGIN Sun_LOGIN FROM ASYMMETRIC KEY Sun_ASYMETRICKEY
GO

GRANT UNSAFE ASSEMBLY TO Sun_LOGIN
GO

--create test database
CREATE DATABASE Sun_DB
GO

USE Sun_DB
GO

--import the assembly 
CREATE ASSEMBLY Sun_ASSEMBLY FROM 'C:\temp\SunClr.dll' WITH PERMISSION_SET=EXTERNAL_ACCESS
GO
 
 --create wrapper function for CLR function Split
CREATE FUNCTION Split(@StringToSplit NVARCHAR(MAX), @SplitOnChars NVARCHAR(MAX) )
RETURNS TABLE(Results nvarchar(max))
AS EXTERNAL NAME Sun_ASSEMBLY.[SunClr_Namespace.SunClr_Class].Split
GO

SELECT * FROM Split('1,2,3,4,5:6:7~8~9',',:~')
GO

--create wrapper function for CLR function ListFiles
CREATE FUNCTION ListFiles(@dirPath NVARCHAR(MAX))
RETURNS TABLE(Name NVARCHAR(MAX), LastWriteTime DATETIME, Type NVARCHAR(MAX))
AS
EXTERNAL NAME Sun_ASSEMBLY.[SunClr_Namespace.SunClr_Class].ListFiles
GO

SELECT * FROM ListFiles('c:\windows')
GO

--create wrapper function for CLR function ListDirs
CREATE FUNCTION ListDirs(@dirPath NVARCHAR(MAX))
RETURNS TABLE(Name NVARCHAR(MAX), LastWriteTime DATETIME, Type NVARCHAR(MAX))
AS
EXTERNAL NAME Sun_ASSEMBLY.[SunClr_Namespace.SunClr_Class].ListDirs
GO

SELECT * FROM ListDirs('c:\windows')
GO

--create wrapper function for CLR function GetDBCount
CREATE FUNCTION GetDBCount() RETURNS INT   
AS EXTERNAL NAME Sun_ASSEMBLY.[SunClr_Namespace.SunClr_Class].GetDBCount;   
GO  

SELECT dbo.GetDBCount(); 
GO

--create wrapper function for CLR aggregate WeightedAvg
CREATE AGGREGATE WeightedAvg (@value int, @weight int) RETURNS int  
EXTERNAL NAME Sun_ASSEMBLY.[SunClr_Namespace.WeightedAvg];  
GO 

DECLARE @TestTable TABLE(weight INT, value INT)
INSERT INTO @TestTable VALUES(1, 400), (3, 100);  

SELECT dbo.WeightedAvg(value, weight) FROM @TestTable
GO

--create wrapper function for CLR procedure GetWindowsLocale
CREATE PROCEDURE GetWindowsLocale
AS EXTERNAL NAME Sun_ASSEMBLY.[SunClr_Namespace.SunClr_Class].GetWindowsLocale;
GO

EXEC GetWindowsLocale;
GO


--create wrapper function for CLR trigger displaying info after a table is dropped
CREATE TRIGGER DropTableTrigger ON  DATABASE AFTER DROP_TABLE
AS EXTERNAL NAME Sun_ASSEMBLY.[SunClr_Namespace.SunClr_Class].DropTableTrigger
GO

CREATE TABLE test(i INT)
GO

DROP TABLE test
GO

--create wrapper function for CLR type Point
CREATE TYPE dbo.Point   
EXTERNAL NAME Sun_ASSEMBLY.[SunClr_Namespace.Point];  
GO

DECLARE @p1 Point = '1,2'
DECLARE @p2 Point = '2,4'
SELECT @p1,@p1.ToString(),@p1.IsNull,@p1.X,@p1.Y
SELECT @p1.Distance(), @p1.DistanceFromXY(1,3),@p1.DistanceFrom(@p2)

--if you wish to drop the assembly, first all wrapper functions, procedures etc. need to be dropped
DROP FUNCTION Split
DROP FUNCTION ListFiles
DROP FUNCTION ListDirs
DROP FUNCTION GetDBCount
DROP AGGREGATE WeightedAvg;
DROP PROCEDURE GetWindowsLocale
DROP TRIGGER DropTableTrigger ON DATABASE
DROP TYPE Point
DROP ASSEMBLY Sun_ASSEMBLY 